#ifndef VectorH //To avoid linker errors, define header's unique "name", so...
#define VectorH //...this header will not be compilled more than one time
//==============================================================================
//------------------------------------------------------------------------------
#include <math.h>
//------------------------------------------------------------------------------
/*=========================< Some auxilary macroses >=========================*/
//Roundings for vectors comparing
#define RoundTo6Digits(d) (double(0.000001*__int64((d+0.0000005)*1000000)))
#define RoundTo9Digits(d) (double(0.000000001*__int64((d+0.0000000005)*1000000000)))
#define RoundTo12Digits(d) (double(0.000000000001*__int64((d+0.0000000000005)*1000000000000)))
#define RoundTo15Digits(d) (double(0.000000000000001*__int64((d+0.0000000000000005)*1000000000000000)))
#define DoubleEqualZero(d) (__int64((d+0.0000000000000005)*1000000000000000)==0)
#define DoubleMoreZero(d) (__int64((d+0.0000000000000005)*1000000000000000)>0)
#define DoubleMoreEqualZero(d) (__int64((d+0.0000000000000005)*1000000000000000)>=0)
#define DoubleLessZero(d) (__int64((d+0.0000000000000005)*1000000000000000)<0)
#define DoubleLessEqualZero(d) (__int64((d+0.0000000000000005)*1000000000000000)<=0)
//Macros for calculating Vector's length
#define LengthOfVector(V) (sqrt(V[0]*V[0]+V[1]*V[1]+V[2]*V[2]))
//Macros for calculating squared value of Vector's length
#define SquareLengthOfVector(V) (V[0]*V[0]+V[1]*V[1]+V[2]*V[2])
//Macros for converting angle from degree to radians
#ifndef DegToRad
 #define DegToRad(Angle) (Angle*0.017453292519943295769236907684886)
#endif
/*========================< VECTOR CLASS DECLARATION >========================*/
typedef class TVector{
double Vxyz[3]; //Vector's coordinates
public:
//Constructors
inline TVector();                          //Default constructor (x=0,y=0,z=0)
inline TVector(double x,double y,double z);//Init with separate coordinates
inline TVector(double Pbeg[3],double Pend[3]);//Specify end and begin points
inline TVector(float Pbeg[3], float Pend[3]); //Specify end and begin points
//Length calculators
inline double SquareLength(); //Faster than Length(); returns square of length
inline double Length();       //Returns vector's length
//Assigning operators
inline void operator ()(double x,double y,double z);//Set coordinates separately
inline void operator = (double NewXYZ[3]);//Copy coordinates from given array
inline void operator = (float NewXYZ[3]); //Copy coordinates from given array
//Easy access to vector's data (emulation: TVector "==" double[3])
inline double& operator [](int XYZ_Index); //Access to x/y/z (0/1/2) coordinates
inline double* operator & (); //Access to variable containing coordinates
//---------------------------< Vector mathematics >---------------------------\\
//Vector's transformation operators
inline bool operator = (double DesiredLength);//Scale vector to specifyed length
inline void operator ! ();                    //Inverse vector's direction
inline void operator +=(TVector Vector);      //Summarize given vector
inline void operator -=(TVector Vector);      //Substract given vector
inline void operator *=(double ScalarValue);  //Multiply by scalar value
inline void operator /=(double ScalarValue);  //Divide by scalar value
//Scalar operators
inline TVector operator *(double ScalarValue);//Multiply by scalar
inline TVector operator /(double ScalarValue);//Divide by scalar
inline  double operator ^(TVector Vector);//Scalar multiplication = cos(V1^V2)
//Vector operators
inline TVector operator +(TVector Vector); //Summarize vectors
inline TVector operator -(TVector Vector); //Substract vectors
inline TVector operator -(); //Inversing operator (necessary in case "a = -b;")
inline TVector operator *(TVector Vector); //Geometrical multiplication
inline TVector operator %(TVector Vector); //Projection of one vector to another
//Boolean operators
inline bool operator ||(TVector Vector);//Check for colinearity
inline bool operator ==(TVector Vector);//Check directions and lengthes equality
inline bool operator !=(TVector Vector);//Check directions or lengthes inequality
inline bool operator > (TVector Vector);//Compare vectors' lengthes
inline bool operator < (TVector Vector);//Compare vectors' lengthes
inline bool operator >=(TVector Vector);//Compare vectors' lengthes
inline bool operator <=(TVector Vector);//Compare vectors' lengthes
inline bool operator ==(double CheckingLength); //Compare vector's length
inline bool operator !=(double CheckingLength); //Compare vector's length
inline bool operator > (double CheckingLength); //Compare vector's length
inline bool operator < (double CheckingLength); //Compare vector's length
inline bool operator >=(double CheckingLength); //Compare vector's length
inline bool operator <=(double CheckingLength); //Compare vector's length
inline void RotateAround(TVector,double); //Rotate around vector by angle

}TVector;

/*========================< VECTOR CLASS DEFINITIONS >========================*/
inline TVector::TVector(){
Vxyz[0]=0;
Vxyz[1]=0;
Vxyz[2]=0;
}
//------------------------------------------------------------------------------
inline TVector::TVector(double x, double y, double z){
Vxyz[0]=x;
Vxyz[1]=y;
Vxyz[2]=z;
}
//------------------------------------------------------------------------------
inline TVector::TVector(double Pbeg[3], double Pend[3]){
Vxyz[0]=Pend[0]-Pbeg[0];
Vxyz[1]=Pend[1]-Pbeg[1];
Vxyz[2]=Pend[2]-Pbeg[2];
}
//------------------------------------------------------------------------------
inline TVector::TVector(float Pbeg[3], float Pend[3]){
Vxyz[0]=Pend[0]-Pbeg[0];
Vxyz[1]=Pend[1]-Pbeg[1];
Vxyz[2]=Pend[2]-Pbeg[2];
}
//------------------------------------------------------------------------------
inline double TVector::SquareLength(){
return SquareLengthOfVector(Vxyz);
}
//------------------------------------------------------------------------------
inline double TVector::Length(){
return LengthOfVector(Vxyz);
}
//------------------------------------------------------------------------------
inline void TVector::operator ()(double x, double y, double z){
Vxyz[0]=x;
Vxyz[1]=y;
Vxyz[2]=z;
}
//------------------------------------------------------------------------------
inline void TVector::operator =(double *NewXYZ){
Vxyz[0]=NewXYZ[0];
Vxyz[1]=NewXYZ[1];
Vxyz[2]=NewXYZ[2];
}
//------------------------------------------------------------------------------
inline void TVector::operator =(float *NewXYZ){
Vxyz[0]=NewXYZ[0];
Vxyz[1]=NewXYZ[1];
Vxyz[2]=NewXYZ[2];
}
//------------------------------------------------------------------------------
inline double& TVector::operator [](int XYZ_Index){
return Vxyz[XYZ_Index];
}
//------------------------------------------------------------------------------
inline double* TVector::operator &(){
return Vxyz;
}
//------------------------------------------------------------------------------
inline bool TVector::operator =(double DesiredLength){
double l = LengthOfVector(Vxyz); if(l==0)return false;
double m=DesiredLength/l;
Vxyz[0]*=m;
Vxyz[1]*=m;
Vxyz[2]*=m;
return true;
}
//------------------------------------------------------------------------------
inline void TVector::operator !(){
Vxyz[0]=-Vxyz[0];
Vxyz[1]=-Vxyz[1];
Vxyz[2]=-Vxyz[2];
}
//------------------------------------------------------------------------------
inline void TVector::operator +=(TVector Vector){
Vxyz[0]+=Vector.Vxyz[0];
Vxyz[1]+=Vector.Vxyz[1];
Vxyz[2]+=Vector.Vxyz[2];
}
//------------------------------------------------------------------------------
inline void TVector::operator -=(TVector Vector){
Vxyz[0]-=Vector.Vxyz[0];
Vxyz[1]-=Vector.Vxyz[1];
Vxyz[2]-=Vector.Vxyz[2];
}
//------------------------------------------------------------------------------
inline void TVector::operator *=(double ScalarValue){
Vxyz[0]*=ScalarValue;
Vxyz[1]*=ScalarValue;
Vxyz[2]*=ScalarValue;
}
//------------------------------------------------------------------------------
inline void TVector::operator /=(double ScalarValue){
Vxyz[0]/=ScalarValue;
Vxyz[1]/=ScalarValue;
Vxyz[2]/=ScalarValue;
}
//------------------------------------------------------------------------------
inline TVector TVector::operator *(double ScalarValue){
return TVector(Vxyz[0]*ScalarValue,Vxyz[1]*ScalarValue,Vxyz[2]*ScalarValue);
}
//------------------------------------------------------------------------------
inline TVector TVector::operator /(double ScalarValue){
return TVector(Vxyz[0]/ScalarValue,Vxyz[1]/ScalarValue,Vxyz[2]/ScalarValue);
}
//------------------------------------------------------------------------------
inline double TVector::operator ^(TVector Vector){
double l1l2=sqrt(SquareLengthOfVector(Vxyz)*SquareLengthOfVector(Vector.Vxyz));
if(l1l2==0)return 1;
return ((Vxyz[0]*Vector.Vxyz[0]+Vxyz[1]*Vector.Vxyz[1]+Vxyz[2]*Vector.Vxyz[2])/
        l1l2);
}
//------------------------------------------------------------------------------
inline TVector TVector::operator +(TVector Vector){
return TVector(Vxyz[0]+Vector.Vxyz[0],
               Vxyz[1]+Vector.Vxyz[1],
               Vxyz[2]+Vector.Vxyz[2]);
}
//------------------------------------------------------------------------------
inline TVector TVector::operator -(TVector Vector){
return TVector(Vxyz[0]-Vector.Vxyz[0],
               Vxyz[1]-Vector.Vxyz[1],
               Vxyz[2]-Vector.Vxyz[2]);
}
//------------------------------------------------------------------------------
inline TVector TVector::operator -(){
return TVector(-Vxyz[0],-Vxyz[1],-Vxyz[2]);
}
//------------------------------------------------------------------------------
inline TVector TVector::operator *(TVector Vector){
return TVector(Vxyz[1]*Vector.Vxyz[2]-Vxyz[2]*Vector.Vxyz[1],
               Vxyz[2]*Vector.Vxyz[0]-Vxyz[0]*Vector.Vxyz[2],
               Vxyz[0]*Vector.Vxyz[1]-Vxyz[1]*Vector.Vxyz[0]);
}
//------------------------------------------------------------------------------
inline TVector TVector::operator %(TVector Vector){
double l=SquareLengthOfVector(Vector.Vxyz);
if(l==0)return TVector(Vxyz[0],Vxyz[1],Vxyz[2]);
double k =
 (Vxyz[0]*Vector.Vxyz[0]+Vxyz[1]*Vector.Vxyz[1]+Vxyz[2]*Vector.Vxyz[2])/l;
return TVector(Vector.Vxyz[0]*k,
               Vector.Vxyz[1]*k,
               Vector.Vxyz[2]*k);
}
//------------------------------------------------------------------------------
inline bool TVector::operator ||(TVector Vector){
if((Vxyz[0]==0)&&(Vxyz[1]==0)&&(Vxyz[2]==0))return true; //0 collinear to any V
double k = ((Vxyz[0]!=0)?(Vector.Vxyz[0]/Vxyz[0]):
            ((Vxyz[1]!=0)?(Vector.Vxyz[1]/Vxyz[1]):(Vector.Vxyz[2]/Vxyz[2])));
return (DoubleEqualZero(Vector.Vxyz[0]-Vxyz[0]*k)&&
        DoubleEqualZero(Vector.Vxyz[1]-Vxyz[1]*k)&&
        DoubleEqualZero(Vector.Vxyz[2]-Vxyz[2]*k)  );
}
//------------------------------------------------------------------------------
inline bool TVector::operator ==(TVector Vector){
return (DoubleEqualZero(Vxyz[0]-Vector.Vxyz[0])&&
        DoubleEqualZero(Vxyz[1]-Vector.Vxyz[1])&&
        DoubleEqualZero(Vxyz[2]-Vector.Vxyz[2])  );
}
//------------------------------------------------------------------------------
inline bool TVector::operator !=(TVector Vector){
return (!(DoubleEqualZero(Vxyz[0]-Vector.Vxyz[0])&&
          DoubleEqualZero(Vxyz[1]-Vector.Vxyz[1])&&
          DoubleEqualZero(Vxyz[2]-Vector.Vxyz[2])  ) );
}
//------------------------------------------------------------------------------
inline bool TVector::operator >(TVector Vector){
return DoubleMoreZero(SquareLengthOfVector(Vxyz)-
                      SquareLengthOfVector(Vector.Vxyz));
}
//------------------------------------------------------------------------------
inline bool TVector::operator <(TVector Vector){
return DoubleLessZero(SquareLengthOfVector(Vxyz)-
                      SquareLengthOfVector(Vector.Vxyz));
}
//------------------------------------------------------------------------------
inline bool TVector::operator >=(TVector Vector){
return DoubleMoreEqualZero(SquareLengthOfVector(Vxyz)-
                           SquareLengthOfVector(Vector.Vxyz));
}
//------------------------------------------------------------------------------
inline bool TVector::operator <=(TVector Vector){
return DoubleLessEqualZero(SquareLengthOfVector(Vxyz)-
                           SquareLengthOfVector(Vector.Vxyz));
}
//------------------------------------------------------------------------------
inline bool TVector::operator ==(double CheckingLength){
return DoubleEqualZero(SquareLengthOfVector(Vxyz)-
                       CheckingLength*CheckingLength);
}
//------------------------------------------------------------------------------
inline bool TVector::operator !=(double CheckingLength){
return (!DoubleEqualZero(SquareLengthOfVector(Vxyz)-
                         CheckingLength*CheckingLength));
}
//------------------------------------------------------------------------------
inline bool TVector::operator >(double CheckingLength){
return DoubleMoreZero(SquareLengthOfVector(Vxyz)-
                      CheckingLength*CheckingLength);
}
//------------------------------------------------------------------------------
inline bool TVector::operator <(double CheckingLength){
return DoubleLessZero(SquareLengthOfVector(Vxyz)-
                      CheckingLength*CheckingLength);
}
//------------------------------------------------------------------------------
inline bool TVector::operator >=(double CheckingLength){
return DoubleMoreEqualZero(SquareLengthOfVector(Vxyz)-
                           CheckingLength*CheckingLength);
}
//------------------------------------------------------------------------------
inline bool TVector::operator <=(double CheckingLength){
return DoubleLessEqualZero(SquareLengthOfVector(Vxyz)-
                           CheckingLength*CheckingLength);
}
//------------------------------------------------------------------------------
inline void TVector::RotateAround(TVector Vector, double AngleInDegrees){
Vector=1;
TVector t = *this * Vector;
*this = t*sin(DegToRad(AngleInDegrees)) +
        (Vector*t)*cos(DegToRad(AngleInDegrees)) +
        *this % Vector;
}
//------------------------------------------------------------------------------



/*======================< QUATERNION CLASS DECLARATION >======================*/
typedef class TQuaternion{
double M[16]; //Matrix
public:
//Constructors
inline TQuaternion();                       //Default constructor
inline void operator =(double*);            //Assign array double[16]
inline void operator =(float*);             //Assign array float[16]
inline double& operator [](int);            //Access to matrix' members
inline double* operator & ();               //Access to matrix
inline TQuaternion operator *(TQuaternion); //Multiplication by quaternion
inline void operator *=(TQuaternion);       //Multiply by another quaternion
inline void Translate(double,double,double);//Translate by given coordinates
inline void Translate(TVector);             //Translate by vector
inline void Scale(double,double,double);    //Scale each axis by given value
inline void Scale(double);                  //Scale all axises equally
inline void Rotate(double,double,double,double);//Rotate by angle around vector
inline void Rotate(double,TVector);         //Rotate by angle around vector
inline TVector operator *(TVector);         //Multiply vector by this quaternion
}TQuaternion;

/*======================< QUATERNION CLASS DEFINITIONS >======================*/
inline TQuaternion::TQuaternion(){
M[0]=1;  M[1]=0;  M[2]=0;  M[3]=0;
M[4]=0;  M[5]=1;  M[6]=0;  M[7]=0;
M[8]=0;  M[9]=0;  M[10]=1; M[11]=0;
M[12]=0; M[13]=0; M[14]=0; M[15]=1;
}
//------------------------------------------------------------------------------
inline void TQuaternion::operator =(double *M16){
M[0]=M16[0];   M[1]=M16[1];   M[2]=M16[2];   M[3]=M16[3];
M[4]=M16[4];   M[5]=M16[5];   M[6]=M16[6];   M[7]=M16[7];
M[8]=M16[8];   M[9]=M16[9];   M[10]=M16[10]; M[11]=M16[11];
M[12]=M16[12]; M[13]=M16[13]; M[14]=M16[14]; M[15]=M16[15];
}
//------------------------------------------------------------------------------
inline void TQuaternion::operator =(float *M16){
M[0]=M16[0];   M[1]=M16[1];   M[2]=M16[2];   M[3]=M16[3];
M[4]=M16[4];   M[5]=M16[5];   M[6]=M16[6];   M[7]=M16[7];
M[8]=M16[8];   M[9]=M16[9];   M[10]=M16[10]; M[11]=M16[11];
M[12]=M16[12]; M[13]=M16[13]; M[14]=M16[14]; M[15]=M16[15];
}
//------------------------------------------------------------------------------
inline double& TQuaternion::operator [](int Index){
return M[Index];
}
//------------------------------------------------------------------------------
inline double* TQuaternion::operator &(){
return M;
}
//------------------------------------------------------------------------------
inline TQuaternion TQuaternion::operator *(TQuaternion Quat){
TQuaternion Result;
Result[0]=Quat[0]*M[0]+Quat[1]*M[4]+Quat[2]*M[8]+Quat[3]*M[12];
Result[1]=Quat[0]*M[1]+Quat[1]*M[5]+Quat[2]*M[9]+Quat[3]*M[13];
Result[2]=Quat[0]*M[2]+Quat[1]*M[6]+Quat[2]*M[10]+Quat[3]*M[14];
Result[3]=Quat[0]*M[3]+Quat[1]*M[7]+Quat[2]*M[11]+Quat[3]*M[15];

Result[4]=Quat[4]*M[0]+Quat[5]*M[4]+Quat[6]*M[8]+Quat[7]*M[12];
Result[5]=Quat[4]*M[1]+Quat[5]*M[5]+Quat[6]*M[9]+Quat[7]*M[13];
Result[6]=Quat[4]*M[2]+Quat[5]*M[6]+Quat[6]*M[10]+Quat[7]*M[14];
Result[7]=Quat[4]*M[3]+Quat[5]*M[7]+Quat[6]*M[11]+Quat[7]*M[15];

Result[8]=Quat[8]*M[0]+Quat[9]*M[4]+Quat[10]*M[8]+Quat[11]*M[12];
Result[9]=Quat[8]*M[1]+Quat[9]*M[5]+Quat[10]*M[9]+Quat[11]*M[13];
Result[10]=Quat[8]*M[2]+Quat[9]*M[6]+Quat[10]*M[10]+Quat[11]*M[14];
Result[11]=Quat[8]*M[3]+Quat[9]*M[7]+Quat[10]*M[11]+Quat[11]*M[15];

Result[12]=Quat[12]*M[0]+Quat[13]*M[4]+Quat[14]*M[8]+Quat[15]*M[12];
Result[13]=Quat[12]*M[1]+Quat[13]*M[5]+Quat[14]*M[9]+Quat[15]*M[13];
Result[14]=Quat[12]*M[2]+Quat[13]*M[6]+Quat[14]*M[10]+Quat[15]*M[14];
Result[15]=Quat[12]*M[3]+Quat[13]*M[7]+Quat[14]*M[11]+Quat[15]*M[15];
return Result;
}
//------------------------------------------------------------------------------
inline void TQuaternion::operator *=(TQuaternion Quat){
TQuaternion Result;
Result[0]=Quat[0]*M[0]+Quat[1]*M[4]+Quat[2]*M[8]+Quat[3]*M[12];
Result[1]=Quat[0]*M[1]+Quat[1]*M[5]+Quat[2]*M[9]+Quat[3]*M[13];
Result[2]=Quat[0]*M[2]+Quat[1]*M[6]+Quat[2]*M[10]+Quat[3]*M[14];
Result[3]=Quat[0]*M[3]+Quat[1]*M[7]+Quat[2]*M[11]+Quat[3]*M[15];

Result[4]=Quat[4]*M[0]+Quat[5]*M[4]+Quat[6]*M[8]+Quat[7]*M[12];
Result[5]=Quat[4]*M[1]+Quat[5]*M[5]+Quat[6]*M[9]+Quat[7]*M[13];
Result[6]=Quat[4]*M[2]+Quat[5]*M[6]+Quat[6]*M[10]+Quat[7]*M[14];
Result[7]=Quat[4]*M[3]+Quat[5]*M[7]+Quat[6]*M[11]+Quat[7]*M[15];

Result[8]=Quat[8]*M[0]+Quat[9]*M[4]+Quat[10]*M[8]+Quat[11]*M[12];
Result[9]=Quat[8]*M[1]+Quat[9]*M[5]+Quat[10]*M[9]+Quat[11]*M[13];
Result[10]=Quat[8]*M[2]+Quat[9]*M[6]+Quat[10]*M[10]+Quat[11]*M[14];
Result[11]=Quat[8]*M[3]+Quat[9]*M[7]+Quat[10]*M[11]+Quat[11]*M[15];

Result[12]=Quat[12]*M[0]+Quat[13]*M[4]+Quat[14]*M[8]+Quat[15]*M[12];
Result[13]=Quat[12]*M[1]+Quat[13]*M[5]+Quat[14]*M[9]+Quat[15]*M[13];
Result[14]=Quat[12]*M[2]+Quat[13]*M[6]+Quat[14]*M[10]+Quat[15]*M[14];
Result[15]=Quat[12]*M[3]+Quat[13]*M[7]+Quat[14]*M[11]+Quat[15]*M[15];
*this = Result;
}
//------------------------------------------------------------------------------
inline void TQuaternion::Translate(double x, double y, double z){
TQuaternion T; T[12]=x; T[13]=y; T[14]=z;
*this *= T;
}
//------------------------------------------------------------------------------
inline void TQuaternion::Translate(TVector V){
TQuaternion T; T[12]=V[0]; T[13]=V[1]; T[14]=V[2];
*this *= T;
}
//------------------------------------------------------------------------------
inline void TQuaternion::Scale(double x, double y, double z){
TQuaternion S; S[0]=x; S[5]=y; S[10]=z;
*this *= S;
}
//------------------------------------------------------------------------------
inline void TQuaternion::Scale(double Value){
TQuaternion S; S[0]=Value; S[5]=Value; S[10]=Value;
*this *= S;
}
//------------------------------------------------------------------------------
inline void TQuaternion::Rotate(double Angle, double x, double y, double z){
double c = cos(DegToRad(Angle));
double s = sin(DegToRad(Angle));
double L = sqrt(x*x+y*y+z*z); if(L==0)return;
x/=L;y/=L;z/=L;
TQuaternion R;
R[0]= x*x*(1-c)+c;   R[1]= y*x*(1-c)+z*s; R[2]= z*x*(1-c)-y*s; R[3]= 0;
R[4]= x*y*(1-c)-z*s; R[5]= y*y*(1-c)+c;   R[6]= z*y*(1-c)+x*s; R[7]= 0;
R[8]= x*z*(1-c)+y*s; R[9]= y*z*(1-c)-x*s; R[10]=z*z*(1-c)+c;   R[11]=0;
R[12]=0;             R[13]=0;             R[14]=0;             R[15]=1;
*this *= R;
}
//------------------------------------------------------------------------------
inline void TQuaternion::Rotate(double Angle, TVector V){
if(!(V=1.0))return;
double c = cos(DegToRad(Angle));
double s = sin(DegToRad(Angle));
TQuaternion R;
R[0]= V[0]*V[0]*(1-c)+c;      R[1]= V[0]*V[1]*(1-c)+V[2]*s; R[2]= V[0]*V[2]*(1-c)-V[1]*s; R[3]= 0;
R[4]= V[1]*V[0]*(1-c)-V[2]*s; R[5]= V[1]*V[1]*(1-c)+c;      R[6]= V[1]*V[2]*(1-c)+V[0]*s; R[7]= 0;
R[8]= V[2]*V[0]*(1-c)+V[1]*s; R[9]= V[2]*V[1]*(1-c)-V[0]*s; R[10]=V[2]*V[2]*(1-c)+c;      R[11]=0;
R[12]=0;                      R[13]=0;                      R[14]=0;                      R[15]=1;
*this *= R;
}
//------------------------------------------------------------------------------
inline TVector TQuaternion::operator *(TVector V){

return TVector(M[0]*V[0]+M[4]*V[1]+M[8]*V[2]+M[12],
               M[1]*V[0]+M[5]*V[1]+M[9]*V[2]+M[13],
               M[2]*V[0]+M[6]*V[1]+M[10]*V[2]+M[14]);
}
//------------------------------------------------------------------------------

//==============================================================================
#endif //End header